pub struct FenwickTree<T: Default + Ord, Op: PrefixOp<T>> { /* private fields */ }Expand description
In a max bit tree or Fenwick Tree, get(i) will return the largest element e that has been added
to the bit tree with set(j, e), where j <= i. Initially all positions have
the value T::default(). Note that a set cannot be ‘undone’ by inserting
a smaller element at the same index.
Time Complexity: O(n) to build a new tree or O(log n) for get() and set() operations,
where n = tree.len().
Implementations