InterpolationTable

Struct InterpolationTable 

pub struct InterpolationTable<F: Fn(f64) -> f64> { /* private fields */ }
Fast lookup table for arbitrary floating point functions. This can be used to e.g., provide fast lookups of distribution values. Input values are sampled with a given precision and results are stored in a vector. During lookup, infimum and supremum of a given value are calculated and the result is interpolated.

Implementations

Source

impl<F: Fn(f64) -> f64> InterpolationTable<F>

pub fn new(min_x: f64, max_x: f64, frac_digits: i32, func: F) -> Self

Create a new InterpolationTable.

Arguments
  • min_x - minimum sample value
  • max_x - maximum sample value
  • frac_digits - number of fraction digits to store in sample
  • func - Function to emulate.

If given value is outside of min_x and max_x, the lookup falls back to applying the function itself. The table size grows with the number of fraction digits. Space Complexity: O(m * 10^n), where m = max_x - min_x and n = frac_digits

pub fn get(&self, x: f64) -> f64

Lookup given value in table, and interpolate the result between the sampled values if necessary. This provides an approximation that is better the more fraction digits are used to generate this table. Time Complexity for lookup: O(1) if min_x <= x < max_x and O(func(x)) otherwise.