pub struct FullMatches<'a, T, C, I>{ /* private fields */ }Expand description
Iterator over tuples of (start, end, distance). Further methods for obtaining the hit alignment path are provided.
Implementations
Sourceimpl<'a, T, C, I> FullMatches<'a, T, C, I>
impl<'a, T, C, I> FullMatches<'a, T, C, I>
Sourcepub fn next_end(&mut self) -> Option<(usize, usize)>
pub fn next_end(&mut self) -> Option<(usize, usize)>
Searches the next match and returns a tuple of end position and distance
if found. This involves no searching for a starting position and is thus
faster than just iterating over FullMatches
Sourcepub fn next_path(
&mut self,
ops: &mut Vec<AlignmentOperation>,
) -> Option<(usize, usize, usize)>
pub fn next_path( &mut self, ops: &mut Vec<AlignmentOperation>, ) -> Option<(usize, usize, usize)>
Searches the next match and returns a tuple of starting position, end position and
distance, or None if no match was found. In addition, the alignment path is added to
ops. Existing data in the vector will be cleared beforehand.
Sourcepub fn next_path_reverse(
&mut self,
ops: &mut Vec<AlignmentOperation>,
) -> Option<(usize, usize, usize)>
pub fn next_path_reverse( &mut self, ops: &mut Vec<AlignmentOperation>, ) -> Option<(usize, usize, usize)>
Like FullMatches::path_reverse(), but the operations will be in reverse order. This
is slightly faster, as the traceback algorithm adds them in reverse order,
and path() needs to reverse them. Existing data in the vector will be cleared
beforehand.
Sourcepub fn next_alignment(&mut self, aln: &mut Alignment) -> bool
pub fn next_alignment(&mut self, aln: &mut Alignment) -> bool
Searches the next match and updates the given Alignment with its position
and alignment path if found. The distance is stored in Alignment::score.
If no next hit is found, false is returned and aln remains unchanged.
Sourcepub fn start(&self) -> Option<usize>
pub fn start(&self) -> Option<usize>
Returns the starting position of the current hit. If the search is finished and no hit was
found, None is returned.
Sourcepub fn path(&self, ops: &mut Vec<AlignmentOperation>) -> Option<usize>
pub fn path(&self, ops: &mut Vec<AlignmentOperation>) -> Option<usize>
Adds the path of the current hit alignment to ops and returns the starting position of
the current hit. If the search is finished and no hit was found, None is returned.
Adds the path of the current hit alignment to ops and returns the starting position of
the current hit. If the search is finished and no hit was found, None is returned.
Existing data in the vector will be cleared beforehand.
Sourcepub fn path_reverse(&self, ops: &mut Vec<AlignmentOperation>) -> Option<usize>
pub fn path_reverse(&self, ops: &mut Vec<AlignmentOperation>) -> Option<usize>
Like FullMatches::path(), but the operations will be in reverse order. This
is slightly faster, as the traceback algorithm adds them in reverse order,
and path() needs to reverse them.
Existing data in the vector will be cleared beforehand.